-- ArcadiaLogos implementation code
--
-- Not fully localized yet (but very close)
--
-- By Iriel (iriel@vigilance-committee.org)

-- Initialize API global table (or use one that's already there)
ArcadiaLogosAPI = ArcadiaLogosAPI or {};

-- Icon path base
local basePath = "Interface\\Addons\\ArcadiaLogos\\Icons\\";
local unknownLogo = basePath .. "NoLogo.tga";

-- State management for montors
local currentMouseoverLogo = nil;
local currentMouseoverName = nil;

-- Main ArcadiaLogos frame
local mainFrame = nil;
-- Flag indicating when mouseover/target events are set
local realmEventsRegistered = nil;

-- Alpha multiplier for 'kill' layer
local KILL_ALPHA = 0.75;
-- Scale for the ugly green UNKNOWN logo
local UNKNOWN_SCALE = 0.75;  -- Scale adjustment for unknown icon

-- Active Settings
local fadeSpeed = 0.5; -- Number of seconds for fade

local activeAlpha = 1.0; -- Alpha when moused over
local inactiveAlpha = 0.6; -- Start alpha of fade (fades to 0.0)

local scaleSetting = 100;  -- Scale setting in percent (from config)
local baseScale = 1.0;     -- Adjusted scale setting value
local scanEnabled = false; -- Whether scanning is enabled
local tempToggled = false; -- Whether scanning is temporarily toggled

local targetEnabled = false; -- Whether target frame indicator is enabled
local randomEnabled  = false; -- Whether random guild selection is enabled
local tooltipEnabled = true; -- Whether tooltip hook is enabled
local tooltipNameEnabled = true; -- Whether tooltip guild name is enabled
local tooltipColorEnabled = true; -- Whether tooltip bg color is enabled

local offsetLength = 3;    -- Offset distance (pixels)
local offsetAngle  = 0;    -- Offset angle (degrees)
local offsetXMul   = 0.0;  -- Derived X offset multiplier (sin angle)
local offsetYMul   = 1.0;  -- Derived Y offset multiplier (cos angle)

-- Bootstrap utility reference (Checked for updates later)
local utils = IrielAddonUtils:GetInstance(ARCADIALOGOS_MESSAGE_CATALOG);

-- Base logo tables, populated during registration
local allianceLogoTable = {};
local hordeLogoTable = {};

-- Logo pack tables:
--   logoPacks    -- keys are logo packs, values are active flags
--   logoPackList -- logo packs in registration order
local logoPacks = {};
local logoPackList = {};

-- Table references, flipped automatically for Horde players
-- Used by the actual comparison code.
local playerSide = "alliance";

-- Debugging function so I can test anywhere.
local randomGuildList;
local function pickRandomGuild(unit)
   if (not randomGuildList) then
      randomGuildList = {};
      for g,v in pairs(allianceLogoTable) do
	 if (v ~= unknownLogo) then
	    table.insert(randomGuildList, g)
	 end
      end
      for g,v in pairs(hordeLogoTable) do
	 if (v ~= unknownLogo) then
	    table.insert(randomGuildList, g)
	 end
      end
      for g,v in pairs(allianceLogoTable) do
	 if (v == unknownLogo) then
	    table.insert(randomGuildList, g)
	 end
      end
      for g,v in pairs(hordeLogoTable) do
	 if (v == unknownLogo) then
	    table.insert(randomGuildList, g)
	 end
      end
   end
   x = string.len(UnitName(unit) or '');
   x = math.mod(x, table.getn(randomGuildList) + 1);
   if (x == 0) then
      return nil;
   end
   return randomGuildList[x];
end

-- Given a unit ID, return logo information for that unit.
-- Parameters
--   unit -- The unit ID to test
-- Returns
--   logo      -- The logo path to use (nil if no logo)
--   logoScale -- The logo scale to apply (nil if no logo)
--   friend    -- Boolean indicating if guild is friend (nil if unknown)
local function ArcadiaLogos_GetUnitLogo(unit)
   if (not UnitExists(unit)) then
      return;
   end
   local guild = GetGuildInfo(unit);
   if ((not guild) and randomEnabled) then
      guild = pickRandomGuild(unit);
   end
   local logo, friend;
   if (guild) then
      logo = allianceLogoTable[guild];
      if (logo) then
	 friend = (playerSide == "alliance");
      else
	 logo = hordeLogoTable[guild];
	 if (logo) then
	    friend = (playerSide == "horde");
	 end
      end
   end
   local logoScale = baseScale;
   if (logo == unknownLogo) then
      logoScale = baseScale * UNKNOWN_SCALE;
   end
   if (logo) then
      return logo, logoScale, friend;
   end
end

-- Move the mouseover icon to track the location of the cursor
-- based on configuration.
function ArcadiaLogos_MoveMouseoverIcon()
   local x,y = GetCursorPosition();

   local ss = this:GetEffectiveScale();
   if (ss == 0) then
      ss = 1;
   end

   this:ClearAllPoints();
   local siz = this:GetWidth();
   local dist = offsetLength / ss + siz / 2;
   this:SetPoint("CENTER", "UIParent", "BOTTOMLEFT", 
		 x/ss + dist * offsetXMul,
		 y/ss + dist * offsetYMul);
end

-- Check the mouseover unit and update the mouseover graphic accordingly
-- Parameters:
--   force -- Normally the logo frame is only updated when something changes
--            set this to force it to be updated regardless.
local function ArcadiaLogos_CheckMouseover(force)
   local logo, name, logoScale, isFriend;

   local disabled = (scanEnabled and tempToggled) or
      ((not scanEnabled) and (not tempToggled));

   if (not disabled) then
      logo, logoScale, isFriend = ArcadiaLogos_GetUnitLogo("mouseover");
      if (logo) then
	 name = UnitName("mouseover");
      end
   end

   if (force or (logo ~= currentMouseoverLogo)) then
      if (not logo) then
	 if (not ArcadiaLogosFrame.fadeTime) then
	    ArcadiaLogosFrame.fadeTime = GetTime();
	 end
	 currentMouseoverLogo = nil;
	 currentMouseoverName = nil;
	 return;
      end

      ArcadiaLogosFrame.fadeTime = nil;
      ArcadiaLogos:SetTexture(logo);
      ArcadiaLogosFrame:SetScale(logoScale);
      ArcadiaLogosFrame:SetAlpha(activeAlpha);
      ArcadiaLogosNoFrame:SetAlpha(activeAlpha * KILL_ALPHA);
      currentMouseoverLogo = logo;
      currentMouseoverName = name;
      -- Icon movement handled by OnShow
      ArcadiaLogosFrame:Show();
      if (isFriend) then
	 ArcadiaLogosNoFrame:Hide();
      else
	 ArcadiaLogosNoFrame:Show();
      end
      return;
   end
end

-- Check the target unit and update the target graphic accordingly
-- Parameters:
local function ArcadiaLogos_CheckTarget()
   local logo, logoScale, isFriend;

   local disabled = (not targetEnabled);

   if (not disabled) then
      logo, logoScale, isFriend = ArcadiaLogos_GetUnitLogo("target");
   end

   if (not logo) then
      ArcadiaTargetFrame:Hide();
      return;
   end

   ArcadiaTargetLogos:SetTexture(logo);
   ArcadiaTargetFrame:Show();
   if (isFriend) then
      ArcadiaTargetNoFrame:Hide();
   else
      ArcadiaTargetNoFrame:Show();
      ArcadiaTargetNoFrame:SetAlpha(KILL_ALPHA);
   end
end

-- Handy utility for hooking
local function pack(...)
  return arg;
end

local function ArcadiaLogos_GameTooltip_AfterSetUnit(self, unit)
   local _,friend;
   if (tooltipEnabled and unit and UnitExists(unit)) then
      guild = GetGuildInfo(unit);
      _,_,friend = ArcadiaLogos_GetUnitLogo(unit)
   end

   if (not guild) then
      return;
   end

   if (tooltipNameEnabled) then
      if (friend == nil) then
	 self:AddLine("<" .. guild .. ">", 0.6, 0.6, 0.6);
      elseif (friend) then
	 self:AddLine("<" .. guild .. ">", 0.4, 1.0, 0.4);
      else 
	 self:AddLine("<" .. guild .. ">", 1.0, 0.2, 0.2);
      end
   end

   if (tooltipColorEnabled) then
      if (friend == false) then
	 self:SetBackdropColor(0.4, 0, 0);
      elseif (friend) then
	 self:SetBackdropColor(0, 0.4, 0);
      end
   end

   self:Show();
end

-- Hook for GameTooltip:SetUnit, to do coloring and guild display if
-- requested
local ArcadiaLogos_GameTooltip_SetUnit_Old;
local ArcadiaLogos_GameTooltip_OnEvent_Old;

local function ArcadiaLogos_GameTooltip_SetUnit(self, ...)
   local unit = arg[1];
   if (tooltipEnabled and unit and UnitExists(unit)) then
      guild = GetGuildInfo(unit);
   end

   if (not guild) then
      return ArcadiaLogos_GameTooltip_SetUnit_Old(self, unpack(arg));
   end
   
   local ret = pack(ArcadiaLogos_GameTooltip_SetUnit_Old(self, unpack(arg)));

   ArcadiaLogos_GameTooltip_AfterSetUnit(self, unit);

   return unpack(ret);
end

-- Check if the game tooltip should be enabled, and adjust hook accordingly.
-- Note that the unhook only happens if it's "safe"
function ArcadiaLogos_CheckTooltip()
   if (tooltipEnabled) then
      if (not ArcadiaLogos_GameTooltip_SetUnit_Old) then
	 ArcadiaLogos_GameTooltip_SetUnit_Old = GameTooltip.SetUnit;
	 GameTooltip.SetUnit = ArcadiaLogos_GameTooltip_SetUnit;
      end

      return;
   end

   -- Try and unhook
   if (ArcadiaLogos_GameTooltip_SetUnit_Old 
       and (GameTooltip.SetUnit == ArcadiaLogos_Gametooltip_SetUnit)) then
      GameTooltip.SetUnit = ArcadiaLogos_GameTooltip_SetUnit_Old ;
      ArcadiaLogos_GameTooltip_SetUnit_Old = nil;
   end

end

-- Get settings structure from saved variables, creating if if missing
local function ArcadiaLogos_GetSettings()
   local settings = ArcadiaLogos_Settings;
   if (not settings) then
      settings = {};
      ArcadiaLogos_Settings = settings;
   end
   return settings;
end

-- Read settings structure and configure local settings variables from
-- saved variables settings.
local function ArcadiaLogos_CheckSettings()
   local settings = ArcadiaLogos_GetSettings();

   activeAlpha   = settings.activeAlpha     or 1.0;
   if (activeAlpha < 0.01) then
      activeAlpha = 0.01;
   elseif (activeAlpha > 1.0) then
      activeAlpha = 1.0;
   end
   inactiveAlpha = activeAlpha * 2.0/3.0;

   scaleSetting  = settings.scale           or 100
   fadeSpeed     = settings.fadeSpeed       or 0.5;

   if (scaleSetting < 10) then
      scaleSetting = 10;
   elseif (scaleSetting > 1000) then
      scaleSetting = 1000;
   end

   if (fadeSpeed < 0.0) then
      fadeSpeed = 0;
   end

   baseScale     = scaleSetting / 100.0;

   if (settings.enabled == nil) then
      scanEnabled   = true;
   else
      scanEnabled   = settings.enabled;
   end

   if (settings.targetEnabled == nil) then
      targetEnabled   = false;
   else
      targetEnabled   = settings.targetEnabled;
   end

   if (settings.randomEnabled == nil) then
      randomEnabled   = false;
   else
      randomEnabled   = settings.randomEnabled;
   end

   offsetAngle = (settings.angle or 0);
   offsetLength = (settings.offsetLength or 3);

   offsetXMul = math.sin(math.rad(offsetAngle));
   offsetYMul = math.cos(math.rad(offsetAngle));

   if (settings.tooltipName == nil) then
      tooltipNameEnabled   = false;
   else
      tooltipNameEnabled   = settings.tooltipName;
   end
   if (settings.tooltipColor == nil) then
      tooltipColorEnabled   = false;
   else
      tooltipColorEnabled   = settings.tooltipColor;
   end
   tooltipEnabled = tooltipNameEnabled or tooltipColorEnabled;
   ArcadiaLogos_CheckTooltip();
end

-- Add the logos from srcMap into destMap unless they're already there
local function ArcadiaLogos_AddLogos(destMap, srcMap) 
   -- Check we actually HAVE a source map
   if (not srcMap) then
      return;
   end
   -- Now apply each guild from source map to the destination
   for guild, logo in pairs(srcMap) do
      local oldValue = destMap[guild];
      if ((oldValue == nil) or (oldValue == unknownLogo)) then
	 if ((logo == "") or (logo == nil)) then
	    logo = unknownLogo;
	 end
	 destMap[guild] = logo;
      end
   end
end

-- Add the logos from a logo pack to the logo list if active (and set
-- active status in the logoPacks map)
local function ArcadiaLogos_AddPack(logoPack)
   local realm = GetCVar("realmName");
   local active = (logoPack.realmName == realm);

   logoPacks[logoPack] = active;

   -- If this logo pack is active, then add its contents to the
   -- lists
   if (active) then
      ArcadiaLogos_AddLogos(allianceLogoTable, logoPack.allianceLogos);
      ArcadiaLogos_AddLogos(hordeLogoTable, logoPack.hordeLogos);

      if (not realmEventsRegistered) then
	 mainFrame:RegisterEvent("PLAYER_TARGET_CHANGED");
	 realmEventsRegistered = true;
      end
   end

   return active;
end

function ArcadiaLogos_EnableScan(enabledFlag, verbose)
   if (enabledFlag == nil) then
      enabledFlag = true;
   end
   local settings = ArcadiaLogos_GetSettings();

   if (settings.scanEnabled ~= enabledFlag) then
      settings.scanEnabled = enabledFlag;
      ArcadiaLogos_CheckSettings();
      ArcadiaLogos_CheckMouseover();
   end
   if (verbose) then
      if (scanEnabled) then
	 utils:Write("CONFIG_SCAN_ENABLED");
      else
	 utils:Write("CONFIG_SCAN_DISABLED");
      end
   end
end

function ArcadiaLogos_EnableTarget(enabledFlag, verbose)
   if (enabledFlag == nil) then
      enabledFlag = true;
   end
   local settings = ArcadiaLogos_GetSettings();

   if (settings.targetEnabled ~= enabledFlag) then
      settings.targetEnabled = enabledFlag;
      ArcadiaLogos_CheckSettings();
      ArcadiaLogos_CheckTarget();
   end
   if (verbose) then
      if (targetEnabled) then
	 utils:Write("CONFIG_TARGET_ENABLED");
      else
	 utils:Write("CONFIG_TARGET_DISABLED");
      end
   end
end

function ArcadiaLogos_EnableTooltipName(enabledFlag, verbose)
   if (enabledFlag == nil) then
      enabledFlag = true;
   end
   local settings = ArcadiaLogos_GetSettings();

   if (settings.tooltipName ~= enabledFlag) then
      settings.tooltipName = enabledFlag;
      ArcadiaLogos_CheckSettings();
   end
   if (verbose) then
      if (tooltipNameEnabled) then
	 utils:Write("CONFIG_TOOLTIP_NAME_ENABLED");
      else
	 utils:Write("CONFIG_TOOLTIP_NAME_DISABLED");
      end
   end
end

function ArcadiaLogos_EnableTooltipColor(enabledFlag, verbose)
   if (enabledFlag == nil) then
      enabledFlag = true;
   end
   local settings = ArcadiaLogos_GetSettings();

   if (settings.tooltipColor ~= enabledFlag) then
      settings.tooltipColor = enabledFlag;
      ArcadiaLogos_CheckSettings();
   end
   if (verbose) then
      if (tooltipColorEnabled) then
	 utils:Write("CONFIG_TOOLTIP_COLOR_ENABLED");
      else
	 utils:Write("CONFIG_TOOLTIP_COLOR_DISABLED");
      end
   end
end

function ArcadiaLogos_ToggleShow()
   ArcadiaLogos_EnableScan(not scanEnabled);

   if (enabledFlag) then
      utils:WriteRaw("ArcadiaLogos: Mouseover logos enabled");
   else
      utils:WriteRaw("ArcadiaLogos: Mouseover logos disabled");
   end
end

function ArcadiaLogos_ToggleHoldShow()
   tempToggled = not tempToggled;

   ArcadiaLogos_CheckMouseover();
end

function ArcadiaLogos_SetAngle(val)
   local settings = ArcadiaLogos_GetSettings();

   if (val < -360) then
      val = -360;
   elseif (val > 360) then
      val = 360;
   end
   settings.angle = val;

   ArcadiaLogos_CheckSettings();
   ArcadiaLogos_CheckMouseover(true);

   utils:WriteRaw("ArcadiaLogos: Angle now "..settings.angle);
end

function ArcadiaLogos_SetAlpha(val)
   local settings = ArcadiaLogos_GetSettings();

   if (val < 0.01) then
      val = 0.01;
   elseif (val > 1.0) then
      val = 1.0;
   end
   settings.activeAlpha = val;

   ArcadiaLogos_CheckSettings();
   ArcadiaLogos_CheckMouseover(true);

   utils:WriteRaw("ArcadiaLogos: Alpha now "..settings.activeAlpha);
end

function ArcadiaLogos_SetOffset(val)
   local settings = ArcadiaLogos_GetSettings();

   if (val < -1000) then
      val = -1000;
   elseif (val > 1000) then
      val = 1000;
   end
   settings.offsetLength = val;

   ArcadiaLogos_CheckSettings();
   ArcadiaLogos_CheckMouseover(true);

   utils:WriteRaw("ArcadiaLogos: Offset distance now "..settings.offsetLength);
end

function ArcadiaLogos_SetSize(val)
   local settings = ArcadiaLogos_GetSettings();

   if (val < 10) then
      val = 10;
   elseif (val > 1000) then
      val = 1000;
   end
   settings.scale = val;

   ArcadiaLogos_CheckSettings();
   ArcadiaLogos_CheckMouseover(true);

   utils:WriteRaw("ArcadiaLogos: Scale now "..settings.scale);
end

function ArcadiaLogos_SetFadeSpeed(val)
   local settings = ArcadiaLogos_GetSettings();
   if (not settings) then
      settings = {};
      ArcadiaLogos_Settings = settings;
   end

   if (val < 0) then
      val = 0;
   end

   settings.fadeSpeed = val;

   ArcadiaLogos_CheckSettings();

   utils:WriteRaw("ArcadiaLogos: Fade speed now " .. fadeSpeed);
end

function ArcadiaLogos_ChangeSize(amount)
   ArcadiaLogos_SetSize(scaleSetting + amount);
end

function ArcadiaLogos_ShowConfig()
   utils:Write("HEADER_CONFIG");
   if (scanEnabled) then
      utils:Write("CONFIG_SCAN_ENABLED");
   else
      utils:Write("CONFIG_SCAN_DISABLED");
   end
   if (targetEnabled) then
      utils:Write("CONFIG_TARGET_ENABLED");
   else
      utils:Write("CONFIG_TARGET_DISABLED");
   end
   if (tooltipNameEnabled) then
      utils:Write("CONFIG_TOOLTIP_NAME_ENABLED");
   else
      utils:Write("CONFIG_TOOLTIP_NAME_DISABLED");
   end
   if (tooltipColorEnabled) then
      utils:Write("CONFIG_TOOLTIP_COLOR_ENABLED");
   else
      utils:Write("CONFIG_TOOLTIP_COLOR_DISABLED");
   end
   if (randomEnabled) then
      utils:Write("CONFIG_RANDOM_ENABLED");
   end
   utils:Write("CONFIG_SCALE", scaleSetting);
   utils:Write("CONFIG_FADE", fadeSpeed);
   utils:Write("CONFIG_ALPHA", activeAlpha);
   utils:Write("CONFIG_ANGLE", offsetAngle);
   utils:Write("CONFIG_OFFSET", offsetLength);

   local packCount,activeCount = 0, 0;
   for p,a in pairs(logoPacks) do
      packCount = packCount + 1;
      if (a) then activeCount = activeCount + 1; end
   end
   utils:Write("CONFIG_PACKS", packCount, activeCount);
end

function ArcadiaLogos_ShowPacks(activeOnly)
   local count = 0;
   for _, pack in ipairs(logoPackList) do
      local isActive = logoPacks[pack];
      local msgKey = nil;
      if (isActive) then
	 msgKey = "SHOWPACKS_ACTIVE"
      elseif (not activeOnly) then
	 msgKey = "SHOWPACKS_INACTIVE"
      end

      if (msgKey) then
	 if (count == 0) then
	    if (activeOnly) then
	       utils:Write("SHOWPACKS_ACTIVE_HEADER");
	    else
	       utils:Write("SHOWPACKS_HEADER");
	    end
	 end
	 count = count + 1;
	 utils:Write(msgKey, 
		     pack.packName or "Unknown",
		     pack.realmName or "Unknown");
      end
   end

   if (count == 0) then
      if (activeOnly) then
	 utils:Write("SHOWPACKS_NONE_ACTIVE");
      else
	 utils:Write("SHOWPACKS_NONE");
      end
   end
end

local COMMAND_TABLE = {
   ["config"] = {
      msgkey = 'CONFIG',
      action = function() ArcadiaLogos_ShowConfig() end,
   },

   ["enable"] = {
      msgkey = "ENABLE",
      action = function() ArcadiaLogos_EnableScan(true, true) end,
   },

   ["disable"] = {
      msgkey = "DISABLE",
      action = function () ArcadiaLogos_EnableScan(false, true) end,
   },

   ["scale"] = {
      msgkey = "SCALE",
      argtype = "RANGE",
      rangelo = 10,
      rangehi = 1000,
      action = function (scale) ArcadiaLogos_SetSize(scale) end,
   },

   ["fade"] = {
      msgkey = "FADE",
      argtype = "RANGE",
      rangelo = 0,
      rangehi = 60,
      action = function (scale) ArcadiaLogos_SetFadeSpeed(scale) end,
   },

   ["angle"] = {
      msgkey = "ANGLE",
      argtype = "RANGE",
      rangelo = -360,
      rangehi = 360,
      action = function (angle) ArcadiaLogos_SetAngle(angle) end,
   },

   ["alpha"] = {
      msgkey = "ALPHA",
      argtype = "RANGE",
      rangelo = 0.01,
      rangehi = 1.0,
      action = function (angle) ArcadiaLogos_SetAlpha(angle) end,
   },

   ["offset"] = {
      msgkey = "OFFSET",
      argtype = "RANGE",
      rangelo = -1000,
      rangehi = 1000,
      action = function (dist) ArcadiaLogos_SetOffset(dist) end,
   },

   ["target"] = {
      msgkey = "TARGET",
      argtype = "FLAG",
      action = function(val) ArcadiaLogos_EnableTarget(val, true) end,
   },

   ["tooltipname"] = {
      msgkey = "TOOLTIP_NAME",
      argtype = "FLAG",
      action = function(val) ArcadiaLogos_EnableTooltipName(val, true) end,
   },

   ["tooltipcolor"] = {
      msgkey = "TOOLTIP_COLOR",
      argtype = "FLAG",
      action = function(val) ArcadiaLogos_EnableTooltipColor(val, true) end,
   },

   ["reset"] = {
      msgkey = "RESET",
      action = function (rest, util) 
		  ArcadiaLogos_Settings = {};
		  util:Write("OPTIONS_RESET")
		  ArcadiaLogos_CheckSettings();
		  ArcadiaLogos_CheckMouseover(true);
	       end
   },

   ["listpacks"] = {
      msgkey = 'LISTPACKS',
      action = function() ArcadiaLogos_ShowPacks() end,
   },

   ["activepacks"] = {
      msgkey = 'ACTIVEPACKS',
      action = function() ArcadiaLogos_ShowPacks(true) end,
   },

}

-- ###########################################################################
--
-- Event handlers and setup
--
-- ###########################################################################

-- Handle slash commands
local function ArcadiaLogos_Command(msg)
   utils:ParseCommand(COMMAND_TABLE, msg);
end

-- Initialize addon, and register command and events
function ArcadiaLogos_OnLoad()
   this.fadeTime = 0;

   -- Remember frame for future event subscription
   mainFrame = this;
   this:RegisterEvent("ADDON_LOADED");
   this:RegisterEvent("PLAYER_ENTERING_WORLD");
   this:RegisterEvent("UPDATE_MOUSEOVER_UNIT");

   -- Avoid doing any extra work if we're not on Dark Iron

   SlashCmdList["ARCADIALOGOS"] = ArcadiaLogos_Command;
end

-- Check to see which side the player is on
local function ArcadiaLogos_CheckRace()
   local race = UnitRace("player");

   if ((race == "Orc")
       or (race == "Troll") 
	  or (race == "Undead") 
	  or (race == "Tauren")) then
      playerSide = "horde";
   else
      playerSide = "alliance";
   end

   return;
end

-- Event handler, dispatch events to appropriate functions
function ArcadiaLogos_OnEvent(event)
   if (event == "ADDON_LOADED") then
      if (arg1 == "ArcadiaLogos") then
	 -- Update utility reference if there's a more recent one
	 utils = utils:CheckInstance();

	 -- Check settings
	 ArcadiaLogos_CheckSettings();

	 -- For future on-demand loading support
	 if (UnitExists("player")) then
	    ArcadiaLogos_CheckRace();
	 end
      end

      return;
   end

   if (event == "PLAYER_ENTERING_WORLD") then
      ArcadiaLogos_CheckRace();
   end

   if (event == "UPDATE_MOUSEOVER_UNIT") then
      if (UnitExists("mouseover")) then
	 ArcadiaLogos_GameTooltip_AfterSetUnit(GameTooltip, "mouseover")
      end
      
      if ((scanEnabled and tempToggled) or
	  ((not scanEnabled) and (not tempToggled))) then
	 return;
      end
      ArcadiaLogos_CheckMouseover();
      return;
   end

   if (event == "PLAYER_TARGET_CHANGED") then
      if (not targetEnabled) then
	 ArcadiaTargetFrame:Hide();
      else
	 ArcadiaLogos_CheckTarget();
      end
      return;
   end
end

-- Update handler, called only when mouseover logo is visible. will
-- ensure mouseover unit is still the same as it was, and manage fading.
-- (Hides frame on full fade to disable update handler).  Also handles
-- tracking.
function ArcadiaLogos_OnUpdate()
   local name = UnitExists("mouseover") and UnitName("mouseover");
   if (name ~= currentMouseoverName) then
      ArcadiaLogos_CheckMouseover();
   end
   if (this.fadeTime) then
      local elapsed;
      if (fadeSpeed > 0) then
	 elapsed = (GetTime() - this.fadeTime) / fadeSpeed;
      else
	 elapsed = 1.0;
      end
      
      if (elapsed >= 1.0) then
	 this:Hide();
	 return;
      else
	 local newAlpha = inactiveAlpha *  (1.0 - elapsed);
	 if (newAlpha < 0) then
	    newAlpha = 0.0;
	 elseif (newAlpha > 1) then
	    newAlpha = 1.0;
	 end
	 ArcadiaLogosFrame:SetAlpha(newAlpha);
	 ArcadiaLogosNoFrame:SetAlpha(newAlpha * KILL_ALPHA);
      end
   end
   if (this:IsVisible()) then
      ArcadiaLogos_MoveMouseoverIcon();
   end
end

-- ###########################################################################
--
-- API FUNCTIONS
--
-- ###########################################################################

-- Register a new logo pack
--   Parameters:
--     logoPack - The logo pack to register.
--   Returns:
--     isActive - nil if pack was already installed, otherwise the active
--                status of removed pack.
function ArcadiaLogosAPI.RegisterLogoPack(logoPack)
   -- Check if it's already registered
   if (logoPacks[logoPack]) then
      return nil;
   end

   local active = ArcadiaLogos_AddPack(logoPack);
   table.insert(logoPackList, logoPack);

   return active;
end

-- Re-load all of the logo packs - use sparingly.
function ArcadiaLogosAPI.ReloadLogoPacks()
   allianceLogoTable = {};
   hordeLogoTable = {};

   for _, logoPack in ipairs(logoPackList) do
      ArcadiaLogos_AddPack(logoPack);
   end
end

-- Unregister a logo pack.
--   Parameters:
--     logoPack - The logo pack to unregister.
--   Returns:
--     wasActive - nil if pack was not installed, otherwise previous active
--                 status of removed pack.
function ArcadiaLogosAPI.UnregisterLogoPack(logoPack)
   -- Check if it's already registered
   if (not logoPacks[logoPack]) then
      return nil;
   end

   local oldIndex = nil;

   -- Find its key in the logoPackList so it can be removed
   for idx,pack in ipairs(logoPackList) do
      if (pack == logoPack) then
	 oldIndex = idx;
	 break;
      end
   end

   -- Remove it from the list
   if (oldIndex) then
      table.remove(logoPackList, oldIndex);
   end

   -- Clear its active status
   local wasActive = logoPacks[logoPack];
   logoPacks[logoPack] = nil;

   -- If it used to be active then issue a reload
   if (wasActive) then
      ArcadiaLogosAPI.ReloadLogoPacks()
   end

   return wasActive;
end
